@@ -1,7 +1,7 @@
 /*
  * tcpd.h	Main header file.
  *
- * 		$Id: tcpd.h,v 1.7 2000/05/15 12:40:58 misiek Exp $
+ * 		$Id: tcpd.h,v 1.8 2000/09/28 16:20:41 baggins Exp $
  * 
  * 		This program is free software; you can redistribute it and/or
  * 		modify it under the terms of the BSD style license (see
@@ -34,7 +34,8 @@ struct request_info {
 	char dst_name[TCPD_HOSTNAMELEN];
 	char service[TCPD_STRINGLEN];
 	char user[TCPD_STRINGLEN];
-	char wrongdns;
+	char src_wrongdns;
+	char dst_wrongdns;
 	char error;
 	char udp;
 	char userresolved;
