@@ -14,7 +14,7 @@
 #endif
 
 #include "rcsid.h"
-RCSID(PKG_VER "$Id: libwrap.c,v 1.17 2000/10/01 21:19:57 baggins Exp $")
+RCSID(PKG_VER "$Id: libwrap.c,v 1.20 2000/10/07 16:50:35 baggins Exp $")
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -41,6 +41,8 @@ struct __service_options {
 	int deny_severity;
 	int ident;
 	int defaultrule;
+	int rule;
+	int lineno;
 	int ignore_src_dns_fake;
 	int ignore_dst_dns_fake;
 };
@@ -465,8 +467,8 @@ static int host_match(struct request_info *req, char *strp, char **left)
  */
 
 static struct request_info *sreq;
-static struct __service_options ALL_service = {-1, -1, -1, -1, -1, -1, -1};
-static struct __service_options current_service = {-1, -1, -1, -1, -1, -1, -1};
+static struct __service_options ALL_service = {-1, -1, -1, -1, -1, -1, -1, -1, -1};
+static struct __service_options current_service = {-1, -1, -1, -1, -1, -1, -1, -1, -1};
 static struct __service_options *sservicep = &ALL_service;
 
 static int set_syslogallow(char *s)
@@ -558,14 +560,10 @@ static int hosts_access0(FILE *fp, struct request_info *req)
 {
 	char buff[TCPD_BUFLEN];
 	char *p, *q, *r, *service;
-#define SERVICE_ALL 2
 #define SERVICE_IN  1
-#define SERVICE_OTHER 0
-	int in_service = SERVICE_ALL;
-	char service_name[TCPD_STRINGLEN];
-	int result = defaultallow;
+#define SERVICE_OUT 0
+	int in_service = SERVICE_OUT;
 
-	*service_name = '\0';
 	while(fgets(buff, sizeof(buff), fp) != NULL) {
 		req->lineno++;
 		p = buff + strlen(buff) -1;
@@ -577,7 +575,7 @@ static int hosts_access0(FILE *fp, struct request_info *req)
 		if (*p == '\0')
 			continue;
 		if (tcpd_word(p, "ALLOW", &q)) {
-			if (in_service != SERVICE_OTHER) {
+			if (in_service != SERVICE_OUT) {
 				tcpd_separator(&q);
 				switch(host_match(req, q, &r)) {
 				case TCPD_MATCH:
@@ -587,7 +585,9 @@ static int hosts_access0(FILE *fp, struct request_info *req)
 					tcpd_to_nextseparator(&r);
 					if (tcpd_exec_option(req) == TCPD_ERROR)
 						return TCPD_ERROR; /* parse error */
-					result = TCPD_ALLOW;
+					sservicep->lineno = req->lineno;
+					sservicep->rule = TCPD_ALLOW;
+					in_service = SERVICE_OUT;
 				case TCPD_CONTINUE:
 					continue;
 				case TCPD_ERROR:
@@ -598,7 +598,7 @@ static int hosts_access0(FILE *fp, struct request_info *req)
 			continue;
 		}
 		if (tcpd_word(p, "DENY", &q)) {
-			if (in_service != SERVICE_OTHER) {
+			if (in_service != SERVICE_OUT) {
 				tcpd_separator(&q);
 				switch(host_match(req, q, &r)) {
 				case TCPD_MATCH:
@@ -608,7 +608,9 @@ static int hosts_access0(FILE *fp, struct request_info *req)
 					tcpd_to_nextseparator(&r);
 					if (tcpd_exec_option(req) == TCPD_ERROR)
 						return TCPD_ERROR; /* parse error */
-					result = TCPD_DENY;
+					sservicep->lineno = req->lineno;
+					sservicep->rule = TCPD_DENY;
+					in_service = SERVICE_OUT;
 				case TCPD_CONTINUE:
 					continue;
 				case TCPD_ERROR:
@@ -638,29 +640,24 @@ static int hosts_access0(FILE *fp, struct request_info *req)
 				else
 					return TCPD_ERROR;
 			}
-			/* Next service? */
-			if (in_service != SERVICE_IN ||
-					!strcasecmp(p, service_name)) {
-				in_service = SERVICE_IN;
-				strncpy(service_name, p, sizeof(service_name));
-			}
+			in_service = SERVICE_IN;
 			SPACESKIP(q);
 			if (tcpd_parse_option(req, q, 0) == TCPD_ERROR)
 				return TCPD_ERROR; /* parse error */
 			continue;
 		case TCPD_NOMATCH:
 			/* Next service */
-			if (in_service == SERVICE_IN) {
-				in_service = SERVICE_OTHER;
-				continue;
-			}
+			in_service = SERVICE_OUT;
+			continue;
 		default:
 			break;
 		}
-		in_service = SERVICE_OTHER;
+		in_service = SERVICE_OUT;
 	}
 	req->lineno = -1;
-	return result;
+	return TCPD_ALLOW; /* IT DOES NOT MATTER HERE! */
+			/* Just return something != TCPD_ERROR */
+			/* The real decision will be made later */
 }
 
 int hosts_access(struct request_info *req)
@@ -680,6 +677,7 @@ int hosts_access(struct request_info *req)
 
 #define SET_VAR(g,a,c) ((((a)<0)&&((c)<0)) ? g : (((c)<0) ? a : c))
 
+	req->lineno = SET_VAR(req->lineno, ALL_service.lineno, current_service.lineno);
 	req->refuse_wait = SET_VAR(req->refuse_wait, ALL_service.refuse_wait, current_service.refuse_wait);
 	req->rfc1413_timeout = SET_VAR(req->rfc1413_timeout, ALL_service.rfc1413_timeout, current_service.rfc1413_timeout);
 	allow_severity = SET_VAR(allow_severity, ALL_service.allow_severity, current_service.allow_severity);
@@ -689,13 +687,13 @@ int hosts_access(struct request_info *req)
 	ignore_src_dns_fake =SET_VAR(ignore_src_dns_fake, ALL_service.ignore_src_dns_fake, current_service.ignore_src_dns_fake);
 	ignore_dst_dns_fake = SET_VAR(ignore_dst_dns_fake, ALL_service.ignore_dst_dns_fake, current_service.ignore_dst_dns_fake);
 
-	if (ret == TCPD_ALLOW && !ignore_src_dns_fake && req->src_wrongdns)
-		ret = TCPD_DENY;
-	else
-	if (ret == TCPD_ALLOW && !ignore_dst_dns_fake && req->dst_wrongdns)
-		ret = TCPD_DENY;
-	else
-	if (ret == TCPD_ERROR) {
+	if (ret != TCPD_ERROR) {
+		ret = SET_VAR(defaultallow, ALL_service.rule, current_service.rule);
+		if ((ret == TCPD_ALLOW) &&
+		    ((!ignore_src_dns_fake && req->src_wrongdns) ||
+		     (!ignore_dst_dns_fake && req->dst_wrongdns)))
+			ret = TCPD_DENY;
+	} else {
 		req->error = -1;
 		ret = TCPD_DENY;
 	}
