@@ -14,7 +14,7 @@
 #endif
 
 #include "rcsid.h"
-RCSID("$Id: refuse.c,v 1.3 2000/02/24 15:07:39 misiek Exp $")
+RCSID("$Id: refuse.c,v 1.6 2000/09/30 22:02:51 baggins Exp $")
 
 #include <stdio.h>
 #include <unistd.h>
@@ -39,14 +39,32 @@ void refuse(struct request_info *req)
 	tcpd_satostr(s1, sizeof s1, &req->src);
 	tcpd_satostr(s2, sizeof s2, &req->dst);
 
-	syslog(deny_severity, "refused connect from %s%s%s[%s] to %s@%s[%s] at line %d%s%s",
+	syslog(deny_severity, "refused connect from %s%s%s[%s] to %s@%s[%s] %s %s %s",
 		req->user, req->user[0] == '\0' ? "": "@",
 		req->src_name, s1,
 		req->service,
 		req->dst_name, s2,
-		req->lineno,
-	    req->error < 0 ? " error": "",
-		req->wrongdns ? " src/dst DNS wrong(fake?)" : "");
+		tcpd_lineno(req),
+		req->error < 0 ? "error": "",
+		req->src_wrongdns ? " src DNS wrong(fake?)" : 
+		(req->dst_wrongdns ? " dst DNS wrong(fake?)" : ""));
 	clean_exit(req);
 }
 
+void tcpd_refuse2(struct request_info *req)
+{
+	char s1[TCPD_STRINGLEN];
+	char s2[TCPD_STRINGLEN];
+
+	tcpd_satostr(s1, sizeof(s1), &req->src);
+	tcpd_satostr(s2, sizeof(s2), &req->dst);
+
+	syslog(deny_severity, "refused connect from %s%s%s[%s] to %s@%s[%s] %s : exec string error: %s",
+		req->user, req->user[0] == '\0' ? "": "@",
+		req->src_name, s1,
+		req->service,
+		req->dst_name, s2,
+		tcpd_lineno(req),
+		req->commands);
+	clean_exit(req);
+}
