@@ -14,7 +14,7 @@
 #endif
 
 #include "rcsid.h"
-RCSID("$Id: tcpd_allowlog.c,v 1.3 2000/02/24 15:07:39 misiek Exp $")
+RCSID("$Id: tcpd_allowlog.c,v 1.5 2000/09/28 16:24:06 baggins Exp $")
 
 #include <stdio.h>
 #include <unistd.h>
@@ -29,6 +29,7 @@ RCSID("$Id: tcpd_allowlog.c,v 1.3 2000/02/24 15:07:39 misiek Exp $")
 #include <syslog.h>
 
 #include "tcpd.h"
+#include "tcpd_local.h"
 
 void tcpd_allowlog(struct request_info *req)
 {
@@ -38,11 +39,12 @@ void tcpd_allowlog(struct request_info *req)
 	tcpd_satostr(s1, sizeof(s1), &req->src);
 	tcpd_satostr(s2, sizeof(s2), &req->dst);
 
-	syslog(allow_severity, "allowed connect from %s%s%s[%s] to %s@%s[%s] at line %d%s",
+	syslog(allow_severity, "allowed connect from %s%s%s[%s] to %s@%s[%s] %s %s",
 		req->user, req->user[0] == '\0' ? "": "@",
 		req->src_name, s1,
 		req->service,
 		req->dst_name, s2,
-		req->lineno,
-		req->wrongdns ? " src/dst DNS wrong(fake?)" : "");
+		tcpd_lineno(req),
+		req->src_wrongdns ? " src DNS wrong(fake?)" : 
+			(req->dst_wrongdns ? " dst DNS wrong(fake?)" : ""));
 }
